==> SD35B_0.02.1_src.tar.gz: Sd35b/decoder/CDPlayer_WIN32.cpp <==
/*
 * CDPlayer_WIN32.cpp - "ALICE SOFT System3.x" compatible scenario decoder SD35
 *
 * Copyright (C) 1999 Atsushi Mizuguchi  All rights reserved.
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA
 * 
 * 
 */

#include "stdafx.h"
#include "CDPlayer_WIN32.h"

using namespace ALSD;
using namespace ALSD_WIN32;
/**********************************************************************
	RXgNV
**********************************************************************/
CCDPlayer_WIN32::CCDPlayer_WIN32()
{
	//
	m_bEnd			= false;
	m_NowTrack		= 0;
	m_NowLoop		= 0;

	// XbhN
	m_hThread = ::CreateThread( NULL, 0, PlayFunc, (void *)this, 0, &m_ThreadID );
}

CCDPlayer_WIN32::~CCDPlayer_WIN32()
{
	m_bEnd = true;
	if( m_hThread ){
		Sleep(500);
		::TerminateThread( m_hThread, 0 );
	}
}

/**********************************************************************
	gbN̎擾
**********************************************************************/
long CCDPlayer_WIN32::GetPlayTrack( ALSD::stCDPTrackInfo *_lpTrack )
{

	return 0;
}

/**********************************************************************
	Đ(gbNOŒ~)
**********************************************************************/
long CCDPlayer_WIN32::Play( long _Track, int _Loop )
{
	m_NowTrack = _Track;
	m_NowLoop  = _Loop;

	return 0;
}

/**********************************************************************
	ĐXbh
**********************************************************************/
unsigned long __stdcall CCDPlayer_WIN32::PlayFunc( void *_lpVoid )
{
	CCDPlayer_WIN32 *lpCDP = (CCDPlayer_WIN32 *)_lpVoid;

	// foCX̃I[v
	MCI_OPEN_PARMS mciOpenParms;
	memset( &mciOpenParms, 0, sizeof( mciOpenParms ));
	mciOpenParms.lpstrDeviceType = "cdaudio";

	if( mciSendCommand( NULL, MCI_OPEN,	MCI_OPEN_TYPE, 
						(DWORD)(LPVOID)&mciOpenParms )  ){
		return -1;
	}
	unsigned short DeviceID = mciOpenParms.wDeviceID;

	// ^CtH[}bg̐ݒ
	MCI_SET_PARMS	mciSetParms;
	memset( &mciSetParms, 0, sizeof( mciSetParms ) );
	mciSetParms.dwTimeFormat = MCI_FORMAT_TMSF;
	DWORD dw = ::mciSendCommand( DeviceID, MCI_SET,
								 MCI_SET_TIME_FORMAT,
								 (DWORD)(LPVOID)&mciSetParms);

	MCI_STATUS_PARMS mciStatusParms;
	memset( &mciStatusParms, 0, sizeof( mciStatusParms ) );
	long track, loop;
	long NowTrack = 0;
	while( !lpCDP->IsDestroy() ){
		Sleep( 100 );
		// bN
		//
		track = lpCDP->GetNowTrack();
		loop  = lpCDP->GetNowLoop();

		// ĐԊmF
		mciStatusParms.dwItem = MCI_STATUS_MODE;
		DWORD dw = ::mciSendCommand( DeviceID, MCI_STATUS,
									 MCI_STATUS_ITEM ,
									 (DWORD)(LPVOID)&mciStatusParms );
		//
		if( mciStatusParms.dwReturn == MCI_MODE_STOP ){
			// ~Ԃ...
			if( track != 0 && loop  >= 0 ){
				// Đ
				lpCDP->PlayTrack( DeviceID, track );
				//
				NowTrack = track;
				if( loop == 1 )		lpCDP->SetNowLoop( -1 );
				else if( loop > 0 )	lpCDP->SetNowLoop( loop-1 );
			}
		}else if( mciStatusParms.dwReturn == MCI_MODE_PLAY ){
			// OȂ
			if( track == 0 ){
				// ~
				lpCDP->StopTrack( DeviceID );
			}else {
				// ݂ƈႤgbNIĂ邩H
				if( track != NowTrack ){
					// ʃgbNĐ
					lpCDP->PlayTrack( DeviceID, track );
					//
					NowTrack = track;
					if( loop == 1 )		lpCDP->SetNowLoop( -1 );
					else if( loop > 0 )	lpCDP->SetNowLoop( loop-1 );
				}
			}
		}
	}
	
	lpCDP->StopTrack( DeviceID );

	// foCX̃N[Y
	MCI_GENERIC_PARMS mciGenericParms;
	memset( &mciGenericParms, 0, sizeof( mciGenericParms ));

	::mciSendCommand( MCI_ALL_DEVICE_ID, MCI_CLOSE, 
						  MCI_WAIT, 
						  (DWORD)(LPVOID)&mciGenericParms);


	return 0;
}


/**********************************************************************
	ĐXgbv
**********************************************************************/
void CCDPlayer_WIN32::PlayTrack( unsigned short _DeviceID, long _Track )
{
	MCI_PLAY_PARMS	mciPlayParms;
	memset( &mciPlayParms, 0, sizeof( mciPlayParms ) );

	// play track
	mciPlayParms.dwFrom = MCI_MAKE_TMSF( _Track, 0, 0, 0 );
	mciPlayParms.dwTo	= MCI_MAKE_TMSF((_Track + 1), 0, 0, 0 );
	DWORD dw = ::mciSendCommand( _DeviceID, MCI_PLAY,
								 MCI_FROM | MCI_TO,
								 (DWORD)(LPVOID)&mciPlayParms );
}

void CCDPlayer_WIN32::StopTrack( unsigned short _DeviceID )
{
	// ~
	MCI_GENERIC_PARMS mciGenericParms;
	memset( &mciGenericParms, 0, sizeof( mciGenericParms ) );
	::mciSendCommand( _DeviceID, MCI_STOP,
					  MCI_WAIT, (DWORD)(LPVOID)&mciGenericParms );
}

==> SD35B_0.03.0_src.tar.gz: Sd35b/decoder/CDPlayer_WIN32.cpp <==
/*
 * CDPlayer_WIN32.cpp - "ALICE SOFT System3.x" compatible scenario decoder SD35
 *
 * Copyright (C) 1999 Atsushi Mizuguchi  All rights reserved.
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA
 * 
 * 
 */

#include "stdafx.h"
#include "CDPlayer_WIN32.h"

using namespace ALSD;
using namespace ALSD_WIN32;
/**********************************************************************
	RXgNV
**********************************************************************/
CCDPlayer_WIN32::CCDPlayer_WIN32()
{
	//
	m_bEnd			= false;
	m_NowTrack		= 0;
	m_NowLoop		= 0;

	// XbhN
	m_hThread = ::CreateThread( NULL, 0, PlayFunc, (void *)this, 0, &m_ThreadID );
}

CCDPlayer_WIN32::~CCDPlayer_WIN32()
{
	m_bEnd = true;
	if( m_hThread ){
		Sleep(500);
		::TerminateThread( m_hThread, 0 );
	}
}

/**********************************************************************
	gbN̎擾
**********************************************************************/
long CCDPlayer_WIN32::GetPlayTrack( ALSD::stCDPTrackInfo *_lpTrack )
{

	return 0;
}

/**********************************************************************
	Đ(gbNOŒ~)
**********************************************************************/
long CCDPlayer_WIN32::Play( long _Track, int _Loop )
{
	m_NowTrack = _Track;
	m_NowLoop  = _Loop;

	return 0;
}

/**********************************************************************
	ĐXbh
**********************************************************************/
unsigned long __stdcall CCDPlayer_WIN32::PlayFunc( void *_lpVoid )
{
	CCDPlayer_WIN32 *lpCDP = (CCDPlayer_WIN32 *)_lpVoid;

	// foCX̃I[v
	MCI_OPEN_PARMS mciOpenParms;
	memset( &mciOpenParms, 0, sizeof( mciOpenParms ));
	mciOpenParms.lpstrDeviceType = "cdaudio";

	if( mciSendCommand( NULL, MCI_OPEN,	MCI_OPEN_TYPE, 
						(DWORD)(LPVOID)&mciOpenParms )  ){
		return -1;
	}
	unsigned short DeviceID = mciOpenParms.wDeviceID;

	// ^CtH[}bg̐ݒ
	MCI_SET_PARMS	mciSetParms;
	memset( &mciSetParms, 0, sizeof( mciSetParms ) );
	mciSetParms.dwTimeFormat = MCI_FORMAT_TMSF;
	DWORD dw = ::mciSendCommand( DeviceID, MCI_SET,
								 MCI_SET_TIME_FORMAT,
								 (DWORD)(LPVOID)&mciSetParms);

	MCI_STATUS_PARMS mciStatusParms;
	memset( &mciStatusParms, 0, sizeof( mciStatusParms ) );
	// CD̃gbN擾
	/*
	::mciSendCommand( DeviceID, MCI_STATUS,
					 MCI_STATUS_NUMBER_OF_TRACKS,
					 (DWORD)(LPVOID)&mciStatusParms );
	m_LastTrack = mciStatusParms.dwReturn;
	*/

	//
	long track, loop;
	long NowTrack = 0;
	while( !lpCDP->IsDestroy() ){
		Sleep( 100 );
		// bN
		//
		track = lpCDP->GetNowTrack();
		loop  = lpCDP->GetNowLoop();

		// ĐԊmF
		mciStatusParms.dwItem = MCI_STATUS_MODE;
		DWORD dw = ::mciSendCommand( DeviceID, MCI_STATUS,
									 MCI_STATUS_ITEM ,
									 (DWORD)(LPVOID)&mciStatusParms );
		//
		if( mciStatusParms.dwReturn == MCI_MODE_STOP ){
			// ~Ԃ...
			if( track != 0 && loop  >= 0 ){
				// Đ
				lpCDP->PlayTrack( DeviceID, track );
				//
				NowTrack = track;
				if( loop == 1 )		lpCDP->SetNowLoop( -1 );
				else if( loop > 0 )	lpCDP->SetNowLoop( loop-1 );
			}
		}else if( mciStatusParms.dwReturn == MCI_MODE_PLAY ){
			// OȂ
			if( track == 0 ){
				// ~
				lpCDP->StopTrack( DeviceID );
			}else {
				// ݂ƈႤgbNIĂ邩H
				if( track != NowTrack ){
					// ʃgbNĐ
					lpCDP->PlayTrack( DeviceID, track );
					//
					NowTrack = track;
					if( loop == 1 )		lpCDP->SetNowLoop( -1 );
					else if( loop > 0 )	lpCDP->SetNowLoop( loop-1 );
				}
			}
		}
	}
	
	lpCDP->StopTrack( DeviceID );

	// foCX̃N[Y
	MCI_GENERIC_PARMS mciGenericParms;
	memset( &mciGenericParms, 0, sizeof( mciGenericParms ));

	::mciSendCommand( MCI_ALL_DEVICE_ID, MCI_CLOSE, 
						  MCI_WAIT, 
						  (DWORD)(LPVOID)&mciGenericParms);


	return 0;
}


/**********************************************************************
	ĐXgbv
**********************************************************************/
void CCDPlayer_WIN32::PlayTrack( unsigned short _DeviceID, long _Track )
{
	// wgbN̒𓾂
	MCI_STATUS_PARMS mciStatusParms;
	memset( &mciStatusParms, 0, sizeof( mciStatusParms ) );
	mciStatusParms.dwTrack = _Track;
	mciStatusParms.dwItem = MCI_STATUS_LENGTH;
	DWORD debug = ::mciSendCommand( _DeviceID, MCI_STATUS,
					  MCI_STATUS_ITEM | MCI_TRACK,

					  (DWORD)(LPVOID)&mciStatusParms );
	unsigned char min   = MCI_MSF_MINUTE( mciStatusParms.dwReturn );
	unsigned char sec   = MCI_MSF_SECOND( mciStatusParms.dwReturn );
	unsigned char frame = MCI_MSF_FRAME( mciStatusParms.dwReturn );

	MCI_PLAY_PARMS	mciPlayParms;
	memset( &mciPlayParms, 0, sizeof( mciPlayParms ) );

	// play track
	mciPlayParms.dwFrom = MCI_MAKE_TMSF( _Track, 0, 0, 0 );
	mciPlayParms.dwTo	= MCI_MAKE_TMSF( _Track, min, sec, frame );
	DWORD dw = ::mciSendCommand( _DeviceID, MCI_PLAY,
								 MCI_FROM | MCI_TO,
								 (DWORD)(LPVOID)&mciPlayParms );
}

void CCDPlayer_WIN32::StopTrack( unsigned short _DeviceID )
{
	// ~
	MCI_GENERIC_PARMS mciGenericParms;
	memset( &mciGenericParms, 0, sizeof( mciGenericParms ) );
	::mciSendCommand( _DeviceID, MCI_STOP,
					  MCI_WAIT, (DWORD)(LPVOID)&mciGenericParms );
}
